@@ -6,9 +6,10 @@ import logging
 import os
 import time
 import argparse
-from typing import Union, Tuple, Optional, Type, List
+from typing import Union, Tuple, Optional, Type, List, Any
+from packaging import version
 
-from .. import QtCore, Flowchart, Signal, Slot, QtWidgets
+from .. import QtCore, Flowchart, Signal, Slot, QtWidgets, QtGui
 from .. import log as plottrlog
 from ..data.datadict import DataDictBase
 from ..data.datadict_storage import DDH5Loader
@@ -18,6 +19,7 @@ from ..gui.widgets import MonitorIntervalInput, SnapshotWidget
 from ..node.data_selector import DataSelector
 from ..node.dim_reducer import XYSelector
 from ..node.filter.correct_offset import SubtractAverage
+from ..node.scaleunits import ScaleUnits
 from ..node.grid import DataGridder, GridOption
 from ..node.tools import linearFlowchart
 from ..node.node import Node
@@ -31,7 +33,7 @@ __license__ = 'MIT'
 # TODO: * separate logging window
 
 
-def logger():
+def logger() -> logging.Logger:
     logger = logging.getLogger('plottr.apps.autoplot')
     logger.setLevel(plottrlog.LEVEL)
     return logger
@@ -80,7 +82,7 @@ class UpdateToolBar(QtWidgets.QToolBar):
     #: Signal emitted after each trigger interval
     trigger = Signal()
 
-    def __init__(self, name, parent=None):
+    def __init__(self, name: str, parent: Optional[QtWidgets.QWidget] = None):
         super().__init__(name, parent)
 
         self.monitorInput = MonitorIntervalInput()
@@ -91,8 +93,8 @@ class UpdateToolBar(QtWidgets.QToolBar):
         self.monitor = QtCore.QTimer()
         self.monitor.timeout.connect(self.monitorTriggered)
 
-    Slot()
-    def monitorTriggered(self):
+    @Slot()
+    def monitorTriggered(self) -> None:
         """
         Is called whenever the monitor timer triggers, and emit the
         :attr:`trigger` Signal.
@@ -100,8 +102,8 @@ class UpdateToolBar(QtWidgets.QToolBar):
         logger().debug('Emit trigger')
         self.trigger.emit()
 
-    Slot(int)
-    def setMonitorInterval(self, val: int):
+    @Slot(float)
+    def setMonitorInterval(self, val: float) -> None:
         """
         Start a background timer that is triggered every `val' seconds.
 
@@ -109,12 +111,12 @@ class UpdateToolBar(QtWidgets.QToolBar):
         """
         self.monitor.stop()
         if val > 0:
-            self.monitor.start(val * 1000)
+            self.monitor.start(int(val * 1000))
 
         self.monitorInput.spin.setValue(val)
 
-    Slot()
-    def stop(self):
+    @Slot()
+    def stop(self) -> None:
         """
         Stop the timer.
         """
@@ -127,11 +129,11 @@ class AutoPlotMainWindow(PlotWindow):
     windowClosed = Signal()
 
     def __init__(self, fc: Flowchart,
-                 parent: Union[QtWidgets.QWidget, None] = None,
+                 parent: Optional[QtWidgets.QMainWindow] = None,
                  monitor: bool = False,
-                 monitorInterval: Union[int, None] = None,
-                 loaderName: str = None,
-                 **kwargs):
+                 monitorInterval: Union[float, None] = None,
+                 loaderName: Optional[str] = None,
+                 **kwargs: Any):
 
         super().__init__(parent, fc=fc, **kwargs)
 
@@ -172,11 +174,11 @@ class AutoPlotMainWindow(PlotWindow):
         else:
             self.monitorToolBar = None
 
-    def setMonitorInterval(self, val):
+    def setMonitorInterval(self, val: float) -> None:
         if self.monitorToolBar is not None:
             self.monitorToolBar.setMonitorInterval(val)
 
-    def closeEvent(self, event):
+    def closeEvent(self, event: QtGui.QCloseEvent) -> None:
         """
         When closing the inspectr window, do some house keeping:
         * stop the monitor, if running
@@ -186,15 +188,15 @@ class AutoPlotMainWindow(PlotWindow):
         self.windowClosed.emit()
         return event.accept()
 
-    def showTime(self):
+    def showTime(self) -> None:
         """
         Displays current time and DS info in the status bar.
         """
         tstamp = time.strftime("%Y-%m-%d %H:%M:%S")
         self.status.showMessage(f"loaded: {tstamp}")
 
-    Slot()
-    def refreshData(self):
+    @Slot()
+    def refreshData(self) -> None:
         """
         Refresh the dataset by calling `update' on the dataset loader node.
         """
@@ -207,7 +209,7 @@ class AutoPlotMainWindow(PlotWindow):
                 self.setDefaults(self.loaderNode.outputValues()['dataOut'])
                 self._initialized = True
 
-    def setInput(self, data: DataDictBase, resetDefaults=True):
+    def setInput(self, data: DataDictBase, resetDefaults: bool = True) -> None:
         """
         Set input to the flowchart. Can only be used when no loader node is
         defined.
@@ -220,7 +222,7 @@ class AutoPlotMainWindow(PlotWindow):
                 self.setDefaults(data)
                 self._initialized = True
 
-    def setDefaults(self, data: DataDictBase):
+    def setDefaults(self, data: DataDictBase) -> None:
         """
         try to set some reasonable defaults so there's a plot right away.
         """
@@ -251,8 +253,8 @@ class QCAutoPlotMainWindow(AutoPlotMainWindow):
     """
 
     def __init__(self, fc: Flowchart,
-                 parent: Union[QtWidgets.QWidget, None] = None,
-                 pathAndId: Union[Tuple[str, int], None] = None, **kw):
+                 parent: Optional[QtWidgets.QMainWindow] = None,
+                 pathAndId: Optional[Tuple[str, int]] = None, **kw: Any):
 
         super().__init__(fc, parent, **kw)
 
@@ -270,6 +272,16 @@ class QCAutoPlotMainWindow(AutoPlotMainWindow):
             self.setDefaults(self.loaderNode.outputValues()['dataOut'])
             self._initialized = True
 
+    def setDefaults(self, data: DataDictBase) -> None:
+        super().setDefaults(data)
+        import qcodes as qc
+        qcodes_support = (version.parse(qc.__version__) >=
+                          version.parse("0.20.0"))
+        if data.meta_val('qcodes_shape') is not None and qcodes_support:
+            self.fc.nodes()['Grid'].grid = GridOption.metadataShape, {}
+        else:
+            self.fc.nodes()['Grid'].grid = GridOption.guessShape, {}
+
 
 def autoplotQcodesDataset(log: bool = False,
                           pathAndId: Union[Tuple[str, int], None] = None) \
@@ -288,6 +300,7 @@ def autoplotQcodesDataset(log: bool = False,
         ('Grid', DataGridder),
         ('Dimension assignment', XYSelector),
         ('Subtract average', SubtractAverage),
+        ('Scale units', ScaleUnits),
         ('plot', PlotNode)
     )
 
@@ -320,25 +333,25 @@ def autoplotDDH5(filepath: str = '', groupname: str = 'data') \
     )
 
     win = AutoPlotMainWindow(fc, loaderName='Data loader', monitor=True,
-                             monitorInterval=2)
+                             monitorInterval=2.0)
     win.show()
 
     fc.nodes()['Data loader'].filepath = filepath
     fc.nodes()['Data loader'].groupname = groupname
     win.refreshData()
-    win.setMonitorInterval(2)
+    win.setMonitorInterval(2.0)
 
     return fc, win
 
 
-def main(f, g):
+def main(f: str, g: str) -> int:
     app = QtWidgets.QApplication([])
     fc, win = autoplotDDH5(f, g)
 
     return app.exec_()
 
 
-def script():
+def script() -> None:
     parser = argparse.ArgumentParser(
         description='plottr autoplot .dd.h5 files.'
     )
