@@ -3,8 +3,8 @@ widgets.py
 
 Common GUI widgets that are re-used across plottr.
 """
-
-from typing import Union, List, Tuple, Optional, Type, Sequence, Dict
+from numpy import rint
+from typing import Union, List, Tuple, Optional, Type, Sequence, Dict, Any
 
 from .tools import dictToTreeWidgetItems
 from plottr import QtCore, Flowchart, QtWidgets, Signal, Slot
@@ -42,24 +42,27 @@ class MonitorIntervalInput(QtWidgets.QWidget):
     Simple form-like widget for entering a monitor/refresh interval.
     Only has a label and a spin-box as input.
 
-    It's signal `intervalChanged(int)' is emitted when the value
+    It's signal `intervalChanged(float)' is emitted when the value
     of the spinbox has changed.
     """
 
-    intervalChanged = Signal(int)
+    intervalChanged = Signal(float)
 
-    def __init__(self, parent=None):
+    def __init__(self, parent: Optional[QtWidgets.QWidget] = None):
         super().__init__(parent)
 
-        self.spin = QtWidgets.QSpinBox()
+        self.spin = QtWidgets.QDoubleSpinBox()
+        self.spin.setSingleStep(0.1)
+        self.spin.setDecimals(1)
+
         layout = QtWidgets.QFormLayout()
         layout.addRow('Refresh interval (s)', self.spin)
         self.setLayout(layout)
 
         self.spin.valueChanged.connect(self.spinValueChanged)
 
-    @Slot(int)
-    def spinValueChanged(self, val):
+    @Slot(float)
+    def spinValueChanged(self, val: float) -> None:
         self.intervalChanged.emit(val)
 
 
@@ -73,7 +76,8 @@ class PlotWindow(QtWidgets.QMainWindow):
 
     plotWidgetClass = MPLAutoPlot
 
-    def __init__(self, parent=None, fc: Flowchart = None, **kw):
+    def __init__(self, parent: Optional[QtWidgets.QMainWindow] = None,
+                 fc: Optional[Flowchart] = None, **kw: Any):
         super().__init__(parent)
 
         self.plot = PlotWidgetContainer(parent=self)
@@ -89,20 +93,22 @@ class PlotWindow(QtWidgets.QMainWindow):
 
         self.setDefaultStyle()
 
-    def setDefaultStyle(self):
+    def setDefaultStyle(self) -> None:
+        scaling = rint(self.logicalDpiX() / 96.0)
+        fontSize = 10*scaling
         self.setStyleSheet(
-            """
-            QToolButton {
-                font: 10px;
-            }
+            f"""
+            QToolButton {{
+                font: {fontSize}px;
+            }}
 
-            QToolBar QCheckBox {
-                font: 10px;
-            }
+            QToolBar QCheckBox {{
+                font: {fontSize}px;
+            }}
             """
         )
 
-    def addNodeWidget(self, node: Node, **kwargs):
+    def addNodeWidget(self, node: Node, **kwargs: Any) -> None:
         """
         Add a node widget as dock.
 
@@ -139,7 +145,7 @@ class PlotWindow(QtWidgets.QMainWindow):
                                     exclude: Sequence[str] = (),
                                     plotNode: str = 'plot',
                                     makePlotWidget: bool = True,
-                                    **kwargs):
+                                    **kwargs: Any) -> None:
         """
         Add all nodes for a flowchart, excluding nodes given in `exclude`.
 
@@ -175,7 +181,7 @@ class PlotWindow(QtWidgets.QMainWindow):
                     self.plot.setPlotWidget(self.plotWidget)
 
 
-def makeFlowchartWithPlotWindow(nodes: List[Tuple[str, Type[Node]]], **kwargs) \
+def makeFlowchartWithPlotWindow(nodes: List[Tuple[str, Type[Node]]], **kwargs: Any) \
         -> Tuple[PlotWindow, Flowchart]:
     nodes.append(('plot', PlotNode))
     fc = linearFlowchart(*nodes)
@@ -185,13 +191,13 @@ def makeFlowchartWithPlotWindow(nodes: List[Tuple[str, Type[Node]]], **kwargs) \
 
 class SnapshotWidget(QtWidgets.QTreeWidget):
 
-    def __init__(self, parent=None):
+    def __init__(self, parent: Optional[QtWidgets.QTreeWidget] = None):
         super().__init__(parent)
 
         self.setHeaderLabels(['Key', 'Value'])
         self.setColumnCount(2)
 
-    def loadSnapshot(self, snapshotDict : Optional[dict]):
+    def loadSnapshot(self, snapshotDict : Optional[dict]) -> None:
         """
         Loads a qcodes DataSet snapshot in the tree view
         """
