@@ -3,7 +3,9 @@ data_selector.py
 
 A node and widget for subselecting from a dataset.
 """
-from typing import List, Tuple, Dict, Any
+from typing import List, Tuple, Dict, Any, Sequence, Optional
+
+import numpy as np
 
 from .node import Node, NodeWidget, updateOption
 from ..data.datadict import DataDictBase, DataDict
@@ -20,7 +22,7 @@ class DataDisplayWidget(NodeWidget):
     Simple Tree widget to show data and their dependencies in the node data.
     """
 
-    def __init__(self, node: Node = None):
+    def __init__(self, node: Optional[Node] = None):
         self.icon = get_dataColumnsIcon()
         super().__init__(embedWidgetClass=DataSelectionWidget)
         assert self.widget is not None
@@ -34,7 +36,7 @@ class DataDisplayWidget(NodeWidget):
         self.widget.dataSelectionMade.connect(
             lambda x: self.signalOption('selectedData'))
 
-    def setSelected(self, vals: List[str]):
+    def setSelected(self, vals: Sequence[str]) -> None:
         assert self.widget is not None
         self.widget.setSelectedData(vals)
         self._updateOptions(vals)
@@ -44,15 +46,16 @@ class DataDisplayWidget(NodeWidget):
         return self.widget.getSelectedData()
 
     def setData(self, structure: DataDictBase,
-                shapes: Dict[str, Tuple[int, ...]], _: Any):
+                shapes: Dict[str, Tuple[int, ...]], _: Any) -> None:
         assert self.widget is not None
         self.widget.setData(structure, shapes)
 
-    def setShape(self, shapes: Dict[str, Tuple[int, ...]]):
+    def setShape(self, shapes: Dict[str, Tuple[int, ...]]) -> None:
         assert self.widget is not None
         self.widget.setShape(shapes)
 
-    def _updateOptions(self, selected):
+    def _updateOptions(self, selected: Sequence[str]) -> None:
+        assert self.widget is not None
         ds = self.widget._dataStructure
         for n, w in self.widget.dataItems.items():
             if selected != [] and ds[n]['axes'] != ds[selected[0]]['axes']:
@@ -81,11 +84,11 @@ class DataSelector(Node):
 
     force_numerical_data = True
 
-    def __init__(self, *arg, **kw):
-        super().__init__(*arg, **kw)
+    def __init__(self, name: str):
+        super().__init__(name)
 
         self._dataStructure = None
-        self.selectedData = []
+        self.selectedData = []  # type: ignore[misc]
 
     # Properties
 
@@ -95,14 +98,14 @@ class DataSelector(Node):
 
     @selectedData.setter  # type: ignore[misc]
     @updateOption('selectedData')
-    def selectedData(self, val: List[str]):
+    def selectedData(self, val: List[str]) -> None:
         if isinstance(val, str):
             val = [val]
         self._selectedData = val
 
     # Data processing
 
-    def validateOptions(self, data):
+    def validateOptions(self, data: DataDictBase) -> bool:
         """
         Validations performed:
         * only compatible data fields can be selected.
@@ -131,7 +134,9 @@ class DataSelector(Node):
                     return False
         return True
 
-    def _reduceData(self, data):
+    def _reduceData(self, data: Optional[DataDictBase]) -> Optional[DataDictBase]:
+        if data is None:
+            return None
         if isinstance(self.selectedData, str):
             dnames = [self.selectedData]
         else:
@@ -142,7 +147,8 @@ class DataSelector(Node):
         ret = data.extract(dnames)
         if self.force_numerical_data:
             for d, _ in ret.data_items():
-                dt = num.largest_numtype(ret.data_vals(d),
+                d_data_vals = ret.data_vals(d)
+                dt = num.largest_numtype(d_data_vals,
                                          include_integers=False)
                 if dt is not None:
                     ret[d]['values'] = ret[d]['values'].astype(dt)
@@ -151,7 +157,7 @@ class DataSelector(Node):
 
         return ret
 
-    def process(self, dataIn: DataDictBase = None):
+    def process(self, dataIn: Optional[DataDictBase] = None) -> Optional[Dict[str, Any]]:
         data = super().process(dataIn=dataIn)
         if data is None:
             return None
@@ -174,8 +180,9 @@ class DataSelector(Node):
 
     # Methods for GUI interaction
 
-    def setupUi(self):
+    def setupUi(self) -> None:
         super().setupUi()
+        assert self.ui is not None
         self.newDataStructure.connect(self.ui.setData)
         self.dataShapesChanged.connect(self.ui.setShape)
 
