@@ -5,7 +5,7 @@ plottr/plot/mpl.py : Tools for plotting with matplotlib.
 import logging
 import io
 from enum import Enum, unique, auto
-from typing import Dict, List, Tuple, Union, cast, Optional
+from typing import Dict, List, Tuple, Union, cast, Type, Optional, Any
 from collections import OrderedDict
 
 # standard scientific computing imports
@@ -155,19 +155,23 @@ symmetric_cmap = cm.get_cmap('bwr')
 
 class SymmetricNorm(colors.Normalize):
     """Color norm that's symmetric and linear around a center value."""
-    def __init__(self, vmin=None, vmax=None, vcenter=0, clip=False):
+    def __init__(self, vmin: Optional[float] = None,
+                 vmax: Optional[float] = None,
+                 vcenter: float = 0,
+                 clip: bool = False):
         super().__init__(vmin, vmax, clip)
         self.vcenter = vcenter
 
-    def __call__(self, value, clip=None):
+    def __call__(self, value: float, clip: Optional[bool] = None) -> np.ma.core.MaskedArray:
         vlim = max(abs(self.vmin-self.vcenter), abs(self.vmax-self.vcenter))
-        self.vmax = vlim+self.vcenter
-        self.vmin = -vlim+self.vcenter
+        self.vmax: float = vlim+self.vcenter
+        self.vmin: float = -vlim+self.vcenter
         return super().__call__(value, clip)
 
 
-def setMplDefaults():
+def setMplDefaults(obj: QtWidgets.QWidget) -> None:
     """Set some reasonable matplotlib defaults for appearance."""
+    scaling = np.rint(obj.logicalDpiX() / 96.0)
 
     rcParams['figure.dpi'] = 300
     rcParams['figure.figsize'] = (4.5, 3)
@@ -176,7 +180,7 @@ def setMplDefaults():
     rcParams['grid.linewidth'] = 0.5
     rcParams['grid.linestyle'] = ':'
     rcParams['font.family'] = 'Arial', 'Helvetica', 'DejaVu Sans'
-    rcParams['font.size'] = 6
+    rcParams['font.size'] = 6 * scaling
     rcParams['lines.markersize'] = 3
     rcParams['lines.linestyle'] = '-'
     rcParams['savefig.transparent'] = False
@@ -189,8 +193,8 @@ def setMplDefaults():
 # 2D plots
 def colorplot2d(ax: Axes, x: np.ndarray, y: np.ndarray, z: np.ndarray,
                 style: PlotType = PlotType.image,
-                axLabels: Tuple[str, str, str] = ('', '', ''),
-                **kw):
+                axLabels: Tuple[Optional[str], Optional[str], Optional[str]] = ('', '', ''),
+                **kw: Any) -> None:
     """make a 2d colorplot. what plot is made, depends on `style`.
     Any of the 2d plot types in :class:`PlotType` works.
 
@@ -199,9 +203,10 @@ def colorplot2d(ax: Axes, x: np.ndarray, y: np.ndarray, z: np.ndarray,
     :param y: y coordinates (meshgrid)
     :param z: z data
     :param style: the plot type
-    :axLabels: labels for the x, y axes, and the colorbar.
+    :param axLabels: labels for the x, y axes, and the colorbar.
 
     all keywords are passed to the actual plotting functions:
+    
     - :attr:`PlotType.image` --
         :func:`plotImage`
     - :attr:`PlotType.colormesh` --
@@ -261,7 +266,7 @@ def colorplot2d(ax: Axes, x: np.ndarray, y: np.ndarray, z: np.ndarray,
 
 
 def ppcolormesh_from_meshgrid(ax: Axes, x: np.ndarray, y: np.ndarray,
-                              z: np.ndarray, **kw) -> Union[AxesImage, None]:
+                              z: np.ndarray, **kw: Any) -> Union[AxesImage, None]:
     r"""Plot a pcolormesh with some reasonable defaults.
     Input are the corresponding arrays from a 2D ``MeshgridDataDict``.
 
@@ -290,7 +295,7 @@ def ppcolormesh_from_meshgrid(ax: Axes, x: np.ndarray, y: np.ndarray,
 
 
 def plotImage(ax: Axes, x: np.ndarray, y: np.ndarray,
-              z: np.ndarray, **kw) -> AxesImage:
+              z: np.ndarray, **kw: Any) -> AxesImage:
     """Plot 2d meshgrid data as image.
 
     :param ax: matplotlib axes to plot the image in.
@@ -348,7 +353,7 @@ def attachColorBar(ax: Axes, im: AxesImage) -> Axes:
 def plot1dTrace(ax: Axes, x: np.ndarray, y: np.ndarray,
                 axLabels: Tuple[Union[None, str], Union[None, str]] = (None, None),
                 curveLabel: Union[None, str] = None,
-                addLegend: bool = False, **kw) -> None:
+                addLegend: bool = False, **kw: Any) -> None:
     """Plot 1D data.
 
     :param ax: Axes to plot into
@@ -408,7 +413,7 @@ class MPLPlot(FCanvas):
     It can be used as any QT widget.
     """
 
-    def __init__(self, parent: QtWidgets.QWidget = None, width: float = 4.0,
+    def __init__(self, parent: Optional[QtWidgets.QWidget] = None, width: float = 4.0,
                  height: float = 3.0, dpi: int = 150, nrows: int = 1,
                  ncols: int = 1):
         """
@@ -434,7 +439,7 @@ class MPLPlot(FCanvas):
         self.clearFig(nrows, ncols)
         self.setParent(parent)
 
-    def autosize(self):
+    def autosize(self) -> None:
         """
         Sets some default spacings/margins.
         :return:
@@ -459,7 +464,7 @@ class MPLPlot(FCanvas):
         :returns: the created axes in the grid
         """
         self.fig.clear()
-        setMplDefaults()
+        setMplDefaults(self)
 
         self.axes = []
         iax = 1
@@ -480,7 +485,7 @@ class MPLPlot(FCanvas):
         self.autosize()
         return self.axes
 
-    def resizeEvent(self, event):
+    def resizeEvent(self, event: QtGui.QResizeEvent) -> None:
         """
         Re-implementation of the widget resizeEvent method.
         Makes sure we resize the plots appropriately.
@@ -488,7 +493,7 @@ class MPLPlot(FCanvas):
         self.autosize()
         super().resizeEvent(event)
 
-    def setTightLayout(self, tight: bool):
+    def setTightLayout(self, tight: bool) -> None:
         """
         Set tight layout mode.
         :param tight: if true, use tight layout for autosizing.
@@ -496,12 +501,12 @@ class MPLPlot(FCanvas):
         self._tightLayout = tight
         self.autosize()
 
-    def setShowInfo(self, show: bool):
+    def setShowInfo(self, show: bool) -> None:
         """Whether to show additional info in the plot"""
         self._showInfo = show
         self.updateInfo()
 
-    def updateInfo(self):
+    def updateInfo(self) -> None:
         if self._infoArtist is not None:
             self._infoArtist.remove()
             self._infoArtist = None
@@ -514,7 +519,7 @@ class MPLPlot(FCanvas):
             )
         self.draw()
 
-    def toClipboard(self):
+    def toClipboard(self) -> None:
         """
         Copy the current canvas to the clipboard.
         """
@@ -525,7 +530,7 @@ class MPLPlot(FCanvas):
             QtGui.QImage.fromData(buf.getvalue()))
         buf.close()
 
-    def setFigureTitle(self, title: str):
+    def setFigureTitle(self, title: str) -> None:
         """Add a title to the figure."""
         self.fig.text(0.5, 0.99, title,
                       horizontalalignment='center',
@@ -533,7 +538,7 @@ class MPLPlot(FCanvas):
                       fontsize='small')
         self.draw()
 
-    def setFigureInfo(self, info: str):
+    def setFigureInfo(self, info: str) -> None:
         """Display an info string in the figure"""
         self._info = info
         self.updateInfo()
@@ -557,28 +562,31 @@ class _MPLPlotWidget(PlotWidget):
     Per default, add a canvas and the matplotlib NavBar.
     """
 
-    def __init__(self, parent=None):
+    def __init__(self, parent: Optional[QtWidgets.QWidget] = None):
         super().__init__(parent=parent)
 
-        setMplDefaults()
+        setMplDefaults(self)
+        scaling = np.rint(self.logicalDpiX() / 96.0)
+        defaultIconSize = 16 * scaling
 
         self.plot = MPLPlot()
         self.mplBar = NavBar(self.plot, self)
         self.addMplBarOptions()
-        self.mplBar.setIconSize(QtCore.QSize(16,16))
+        self.mplBar.setIconSize(QtCore.QSize(defaultIconSize, defaultIconSize))
 
-        self.layout = QtWidgets.QVBoxLayout(self)
-        self.layout.addWidget(self.plot)
-        self.layout.addWidget(self.mplBar)
+        layout = QtWidgets.QVBoxLayout(self)
+        layout.addWidget(self.plot)
+        layout.addWidget(self.mplBar)
+        self.setLayout(layout)
 
-    def setMeta(self, data: DataDictBase):
+    def setMeta(self, data: DataDictBase) -> None:
         if data.has_meta('title'):
             self.plot.setFigureTitle(data.meta_val('title'))
 
         if data.has_meta('info'):
             self.plot.setFigureInfo(data.meta_val('info'))
 
-    def addMplBarOptions(self):
+    def addMplBarOptions(self) -> None:
         tlCheck = QtWidgets.QCheckBox('Tight layout')
         tlCheck.toggled.connect(self.plot.setTightLayout)
 
@@ -608,8 +616,7 @@ class _AutoPlotToolBar(QtWidgets.QToolBar):
     #: signal emitted when the complex data option has been changed
     complexPolarSelected = Signal(bool)
 
-
-    def __init__(self, name: str, parent: QtWidgets.QWidget = None):
+    def __init__(self, name: str, parent: Optional[QtWidgets.QWidget] = None):
         """Constructor for :class:`AutoPlotToolBar`"""
 
         super().__init__(name, parent=parent)
@@ -651,7 +658,7 @@ class _AutoPlotToolBar(QtWidgets.QToolBar):
 
         self.plotComplexPolar = self.addAction('Mag/Phase')
         self.plotComplexPolar.setCheckable(True)
-        self.plotComplexPolar.triggered.connect(self.complexPolarSelected)
+        self.plotComplexPolar.triggered.connect(self._trigger_complex_mag_phase)
 
         self.plotTypeActions = OrderedDict({
             PlotType.multitraces: self.plotasMultiTraces,
@@ -664,7 +671,10 @@ class _AutoPlotToolBar(QtWidgets.QToolBar):
         self._currentPlotType = PlotType.empty
         self._currentlyAllowedPlotTypes: Tuple[PlotType, ...] = ()
 
-    def selectPlotType(self, plotType: PlotType):
+    def _trigger_complex_mag_phase(self, enable: bool) -> None:
+        self.complexPolarSelected.emit(enable)
+
+    def selectPlotType(self, plotType: PlotType) -> None:
         """makes sure that the selected `plotType` is active (checked), all
         others are not active.
 
@@ -686,7 +696,7 @@ class _AutoPlotToolBar(QtWidgets.QToolBar):
             self._currentPlotType = plotType
             self.plotTypeSelected.emit(plotType)
 
-    def setAllowedPlotTypes(self, *args: PlotType):
+    def setAllowedPlotTypes(self, *args: PlotType) -> None:
         """Disable all choices that are not allowed.
         If the current selection is now disabled, instead select the first
         enabled one.
@@ -751,7 +761,7 @@ class AutoPlot(_MPLPlotWidget):
     whereas magnitude and phase are separated into two panels.
     """
 
-    def __init__(self, parent=None):
+    def __init__(self, parent: Optional[QtWidgets.QWidget] = None):
         super().__init__(parent=parent)
 
         self.plotDataType = PlotDataType.unknown
@@ -759,14 +769,14 @@ class AutoPlot(_MPLPlotWidget):
         self.complexRepresentation = ComplexRepresentation.real
         self.complexPreference = ComplexRepresentation.realAndImag
 
-        self.dataType = type(None)
-        self.dataStructure = None
-        self.dataShapes = None
-        self.dataLimits = None
+        self.dataType: Optional[Type[DataDictBase]] = None
+        self.dataStructure: Optional[DataDictBase] = None
+        self.dataShapes: Optional[Dict[str, Tuple[int, ...]]] = None
+        self.dataLimits: Optional[Dict[str, Tuple[float, float]]] = None
 
         # A toolbar for configuring the plot
         self.plotOptionsToolBar = _AutoPlotToolBar('Plot options', self)
-        self.layout.insertWidget(1, self.plotOptionsToolBar)
+        self.layout().insertWidget(1, self.plotOptionsToolBar)
 
         self.plotOptionsToolBar.plotTypeSelected.connect(
             self._plotTypeFromToolBar
@@ -774,14 +784,18 @@ class AutoPlot(_MPLPlotWidget):
         self.plotOptionsToolBar.complexPolarSelected.connect(
             self._complexPreferenceFromToolBar
         )
+        scaling = np.rint(self.logicalDpiX() / 96.0)
+        iconSize = 32 + 8*(scaling - 1)
+        self.plotOptionsToolBar.setIconSize(QtCore.QSize(iconSize, iconSize))
 
-        self.plotOptionsToolBar.setIconSize(QtCore.QSize(32, 32))
-
-        self.setMinimumSize(640, 480)
+        self.setMinimumSize(640*scaling, 480*scaling)
 
     def _analyzeData(self, data: Optional[DataDictBase]) -> Dict[str, bool]:
         """checks data and compares with previous properties."""
-        dataType = type(data)
+        if data is not None:
+            dataType: Optional[Type[DataDictBase]] = type(data)
+        else:
+            dataType = None
 
         if data is None:
             dataStructure = None
@@ -809,7 +823,7 @@ class AutoPlot(_MPLPlotWidget):
 
         return result
 
-    def dataIsComplex(self, dependentName=None):
+    def dataIsComplex(self, dependentName: Optional[str] = None) -> bool:
         """Determine whether our data is complex.
         If dependent_name is not given, check all dependents, return True if any
         of them is complex.
@@ -827,7 +841,7 @@ class AutoPlot(_MPLPlotWidget):
 
         return False
 
-    def setData(self, data: Optional[DataDictBase]):
+    def setData(self, data: Optional[DataDictBase]) -> None:
         """Analyses data and determines whether/what to plot.
 
         :param data: input data
@@ -840,7 +854,7 @@ class AutoPlot(_MPLPlotWidget):
         self._processPlotTypeOptions()
         self._plotData(adjustSize=True)
 
-    def _processPlotTypeOptions(self):
+    def _processPlotTypeOptions(self) -> None:
         """Given the current data type, figure out what the plot options are."""
         if self.plotDataType == PlotDataType.grid2d:
             self.plotOptionsToolBar.setAllowedPlotTypes(
@@ -859,16 +873,16 @@ class AutoPlot(_MPLPlotWidget):
             )
 
         else:
-            self.plotOptionsToolBar.setAllowedPlotTypes([])
+            self.plotOptionsToolBar.setAllowedPlotTypes()
 
     @Slot(PlotType)
-    def _plotTypeFromToolBar(self, plotType: PlotType):
+    def _plotTypeFromToolBar(self, plotType: PlotType) -> None:
         if plotType is not self.plotType:
             self.plotType = plotType
             self._plotData(adjustSize=True)
 
     @Slot(bool)
-    def _complexPreferenceFromToolBar(self, magPhasePreferred):
+    def _complexPreferenceFromToolBar(self, magPhasePreferred: bool) -> None:
         if magPhasePreferred:
             self.complexPreference = ComplexRepresentation.magAndPhase
         else:
@@ -885,7 +899,7 @@ class AutoPlot(_MPLPlotWidget):
         axes = self.plot.clearFig(nrows, ncols, nAxes)
         return axes
 
-    def _plotData(self, adjustSize: bool = False):
+    def _plotData(self, adjustSize: bool = False) -> None:
         """Plot the data using previously determined data and plot types."""
 
         if self.plotDataType is PlotDataType.unknown:
@@ -917,7 +931,7 @@ class AutoPlot(_MPLPlotWidget):
         else:
             logger.info(f"No plot routine defined for {self.plotType}")
             return
-
+        assert self.data is not None
         self.setMeta(self.data)
         if adjustSize:
             self.plot.autosize()
@@ -927,9 +941,10 @@ class AutoPlot(_MPLPlotWidget):
         QtCore.QCoreApplication.processEvents()
 
     # Plotting functions
-    def _plot1dSinglepanel(self):
+    def _plot1dSinglepanel(self) -> None:
+        assert self.data is not None
         xname = self.data.axes()[0]
-        xvals = self.data.data_vals(xname)
+        xvals = np.asanyarray(self.data.data_vals(xname))
         depnames = self.data.dependents()
         depvals = [self.data.data_vals(d) for d in depnames]
 
@@ -942,7 +957,7 @@ class AutoPlot(_MPLPlotWidget):
 
         if len(depvals) > 1:
             ylbl = self.data.label(depnames[0])
-            phlbl = f"Arg({depnames[0]})"
+            phlbl: Optional[str] = f"Arg({depnames[0]})"
         else:
             ylbl = None
             phlbl = None
@@ -955,7 +970,7 @@ class AutoPlot(_MPLPlotWidget):
 
             if self.complexRepresentation in [ComplexRepresentation.real,
                                               ComplexRepresentation.realAndImag]:
-                plot1dTrace(axes[0], xvals, yvals,
+                plot1dTrace(axes[0], xvals, np.asanyarray(yvals),
                             axLabels=(self.data.label(xname), ylbl),
                             curveLabel=self.data.label(yname),
                             addLegend=(yname == depnames[-1]))
@@ -971,14 +986,15 @@ class AutoPlot(_MPLPlotWidget):
                                 curveLabel=f"Arg({yname})",
                                 addLegend=(yname == depnames[-1]))
                 else:
-                    plot1dTrace(axes[0], xvals, yvals,
+                    plot1dTrace(axes[0], xvals, np.asanyarray(yvals),
                                 axLabels=(self.data.label(xname), ylbl),
                                 curveLabel=self.data.label(yname),
                                 addLegend=(yname == depnames[-1]))
 
-    def _plot1dSeparatePanels(self):
+    def _plot1dSeparatePanels(self) -> None:
+        assert self.data is not None
         xname = self.data.axes()[0]
-        xvals = self.data.data_vals(xname)
+        xvals = np.asanyarray(self.data.data_vals(xname))
         depnames = self.data.dependents()
         depvals = [self.data.data_vals(d) for d in depnames]
 
@@ -1005,7 +1021,7 @@ class AutoPlot(_MPLPlotWidget):
 
             if self.complexRepresentation in [ComplexRepresentation.real,
                                               ComplexRepresentation.realAndImag]:
-                plot1dTrace(axes[iax], xvals, yvals,
+                plot1dTrace(axes[iax], xvals, np.asanyarray(yvals),
                             axLabels=(self.data.label(xname), self.data.label(yname)),
                             addLegend=self.dataIsComplex(yname))
                 iax += 1
@@ -1020,16 +1036,17 @@ class AutoPlot(_MPLPlotWidget):
                                           f"Arg({yname})"))
                     iax += 2
                 else:
-                    plot1dTrace(axes[iax], xvals, yvals,
+                    plot1dTrace(axes[iax], xvals, np.asanyarray(yvals),
                                 axLabels=(self.data.label(xname),
                                           self.data.label(yname)))
                     iax += 1
 
-    def _colorplot2d(self):
+    def _colorplot2d(self) -> None:
+        assert self.data is not None
         xname = self.data.axes()[0]
         yname = self.data.axes()[1]
-        xvals = self.data.data_vals(xname)
-        yvals = self.data.data_vals(yname)
+        xvals = np.asanyarray(self.data.data_vals(xname))
+        yvals = np.asanyarray(self.data.data_vals(yname))
         depnames = self.data.dependents()
         depvals = [self.data.data_vals(d) for d in depnames]
 
@@ -1054,7 +1071,7 @@ class AutoPlot(_MPLPlotWidget):
 
             if self.complexRepresentation is ComplexRepresentation.real \
                     or not self.dataIsComplex(zname):
-                colorplot2d(axes[iax], xvals, yvals, zvals.real,
+                colorplot2d(axes[iax], xvals, yvals, np.asanyarray(zvals).real,
                             self.plotType,
                             axLabels=(self.data.label(xname),
                                       self.data.label(yname),
@@ -1062,12 +1079,12 @@ class AutoPlot(_MPLPlotWidget):
                 iax += 1
 
             elif self.complexRepresentation is ComplexRepresentation.realAndImag:
-                colorplot2d(axes[iax], xvals, yvals, zvals.real,
+                colorplot2d(axes[iax], xvals, yvals, np.asanyarray(zvals).real,
                             self.plotType,
                             axLabels=(self.data.label(xname),
                                       self.data.label(yname),
                                       f"Re( {self.data.label(zname)} )"))
-                colorplot2d(axes[iax+1], xvals, yvals, zvals.imag,
+                colorplot2d(axes[iax+1], xvals, yvals, np.asanyarray(zvals).imag,
                             self.plotType,
                             axLabels=(self.data.label(xname),
                                       self.data.label(yname),
@@ -1075,12 +1092,12 @@ class AutoPlot(_MPLPlotWidget):
                 iax += 2
 
             elif self.complexRepresentation is ComplexRepresentation.magAndPhase:
-                colorplot2d(axes[iax], xvals, yvals, np.abs(zvals),
+                colorplot2d(axes[iax], xvals, yvals, np.abs(np.asanyarray(zvals)),
                             self.plotType,
                             axLabels=(self.data.label(xname),
                                       self.data.label(yname),
                                       f"Abs( {self.data.label(zname)} )"))
-                colorplot2d(axes[iax+1], xvals, yvals, np.angle(zvals),
+                colorplot2d(axes[iax+1], xvals, yvals, np.angle(np.asanyarray(zvals)),
                             self.plotType,
                             axLabels=(self.data.label(xname),
                                       self.data.label(yname),
